# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from django import template
from django.utils import timezone
from django.template import loader, Context

register = template.Library()


@register.simple_tag
def include_djangovirtualpos_set_payment_attributes_js(set_payment_attributes_url, sale_code, url_ok, url_nok):
	t = loader.get_template("djangovirtualpos/templatetags/djangovirtualpos_js/djangovirtualpos_js.html")
	replacements = {
		"url_set_payment_attributes": set_payment_attributes_url,
		"sale_code": sale_code,
		"url_ok": url_ok,
		"url_nok": url_nok
	}
	try:
		return t.render(Context(replacements))
	except TypeError:
		return t.render(replacements)


# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2018-04-03 10:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangovirtualpos', '0013_vposredsys_enable_preauth_policy'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='vposredsys',
            name='enable_preauth_policy',
        ),
        migrations.AddField(
            model_name='vposredsys',
            name='operative_type',
            field=models.CharField(choices=[('authorization', 'autorizaci\xf3n'), ('pre-authorization', 'pre-autorizaci\xf3n')], default='authorization', max_length=512, verbose_name='Tipo de operativa'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangovirtualpos', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='vpospaymentoperation',
            name='response_code',
            field=models.CharField(max_length=255, null=True, verbose_name='C\xf3digo de respuesta con estado de aceptaci\xf3n o denegaci\xf3n de la operaci\xf3n.'),
        ),
    ]



# -*- coding: utf-8 -*-

import datetime
from django.conf import settings
from django.utils import timezone
import pytz


########################################################################
########################################################################
# Obtiene la dirección IP del cliente
def get_client_ip(request):
    """
    Obtiene la dirección IP del cliente.
    """
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        return x_forwarded_for.split(',')[0]
    else:
        return request.META.get('REMOTE_ADDR')


def as_server_datetime(_datetime):
    """Localiza la marca de tiempo en función de la zona de tiempo del servidor"""
    SERVERT_PYTZ = pytz.timezone(settings.TIME_ZONE)
    return SERVERT_PYTZ.localize(_datetime)


def localize_datetime(_datetime):
    """Localiza la marca de tiempo en función de la zona de tiempo del servidor. Sólo y exclusivamente si no está localizada ya."""
    if timezone.is_naive(_datetime):
        return as_server_datetime(_datetime)
    return _datetime


def localize_datetime_from_format(str_datetime, datetime_format="%Y-%m-%d %H:%M"):
    _datetime = datetime.datetime.strptime(str_datetime, datetime_format)
    return localize_datetime(_datetime)


########################################################################
########################################################################


def dictlist(node):
    res = {}
    res[node.tag] = []
    xmltodict(node, res[node.tag])
    reply = {}
    reply[node.tag] = {'value': res[node.tag], 'attribs': node.attrib, 'tail': node.tail}

    return reply


def xmltodict(node, res):
    rep = {}
    if len(node):
        # n = 0
        for n in list(node):
            rep[node.tag] = []
            value = xmltodict(n, rep[node.tag])
            if len(n):
                value = {'value': rep[node.tag], 'attributes': n.attrib, 'tail': n.tail}
                res.append({n.tag: value})
            else:
                res.append(rep[node.tag][0])
    else:
        value = {}
        value = {'value': node.text, 'attributes': node.attrib, 'tail': node.tail}

        res.append({node.tag: value})

    return




from __future__ import unicode_literals

from django.conf import settings
from django.conf.urls import url, include
from django.conf.urls.static import static
from django.contrib import admin


from views import confirm_payment

urlpatterns = [
    url(r'^confirm/$', confirm_payment, name="confirm_payment")
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-28 16:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangovirtualpos', '0008_auto_20170321_1437'),
    ]

    operations = [
        migrations.CreateModel(
            name='VPOSRefundOperation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=6, verbose_name='Cantidad de la devoluci\xf3n')),
                ('description', models.CharField(max_length=512, verbose_name='Descripci\xf3n de la devoluci\xf3n')),
                ('operation_number', models.CharField(max_length=255, verbose_name='N\xfamero de operaci\xf3n')),
                ('confirmation_code', models.CharField(max_length=255, null=True, verbose_name='C\xf3digo de confirmaci\xf3n enviado por el banco.')),
                ('status', models.CharField(choices=[('completed', 'Completed'), ('failed', 'Failed')], max_length=64, verbose_name='Estado de la devoluci\xf3n')),
                ('creation_datetime', models.DateTimeField(verbose_name='Fecha de creaci\xf3n del objeto')),
                ('last_update_datetime', models.DateTimeField(verbose_name='Fecha de \xfaltima actualizaci\xf3n del objeto')),
            ],
        ),
        migrations.AlterField(
            model_name='vpospaymentoperation',
            name='status',
            field=models.CharField(choices=[('pending', 'Pending'), ('completed', 'Completed'), ('failed', 'Failed'), ('partially_refunded', 'Partially Refunded'), ('completely_refunded', 'Completely Refunded')], max_length=64, verbose_name='Estado del pago'),
        ),
        migrations.AddField(
            model_name='vposrefundoperation',
            name='payment',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='refund_operation', to='djangovirtualpos.VPOSPaymentOperation'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-29 12:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangovirtualpos', '0009_vpsrefundoperation'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='vposrefundoperation',
            name='confirmation_code',
        ),
        migrations.AlterField(
            model_name='vposrefundoperation',
            name='status',
            field=models.CharField(choices=[('pending', 'Pending'), ('completed', 'Completed'), ('failed', 'Failed')], max_length=64, verbose_name='Estado de la devoluci\xf3n'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2018-02-09 12:22
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangovirtualpos', '0011_auto_20170404_1424'),
    ]

    operations = [
        migrations.CreateModel(
            name='VPOSBitpay',
            fields=[
                ('parent', models.OneToOneField(db_column='vpos_id', on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='+', serialize=False, to='djangovirtualpos.VirtualPointOfSale')),
                ('testing_api_key', models.CharField(blank=True, max_length=512, null=True, verbose_name='API Key de Bitpay para entorno de test')),
                ('production_api_key', models.CharField(max_length=512, verbose_name='API Key de Bitpay para entorno de producci\xf3n')),
                ('currency', models.CharField(choices=[('EUR', 'Euro'), ('USD', 'Dolares'), ('BTC', 'Bitcoin')], default='EUR', max_length=3, verbose_name='Moneda (EUR, USD, BTC)')),
                ('transaction_speed', models.CharField(choices=[('high', 'Alta'), ('medium', 'Media'), ('low', 'Baja')], default='medium', max_length=10, verbose_name='Velocidad de la operaci\xf3n')),
                ('notification_url', models.URLField(verbose_name='Url notificaciones actualizaci\xf3n estados (https)')),
                ('operation_number_prefix', models.CharField(blank=True, max_length=20, null=True, verbose_name='Prefijo del n\xfamero de operaci\xf3n')),
            ],
            bases=('djangovirtualpos.virtualpointofsale',),
        ),
        migrations.AlterField(
            model_name='virtualpointofsale',
            name='type',
            field=models.CharField(choices=[('ceca', 'TPV Virtual - Confederaci\xf3n Espa\xf1ola de Cajas de Ahorros (CECA)'), ('paypal', 'Paypal'), ('redsys', 'TPV Redsys'), ('santanderelavon', 'TPV Santander Elavon'), ('bitpay', 'TPV Bitpay')], default='', max_length=16, verbose_name='Tipo de TPV'),
        ),
        migrations.AlterField(
            model_name='vpospaymentoperation',
            name='type',
            field=models.CharField(choices=[('ceca', 'TPV Virtual - Confederaci\xf3n Espa\xf1ola de Cajas de Ahorros (CECA)'), ('paypal', 'Paypal'), ('redsys', 'TPV Redsys'), ('santanderelavon', 'TPV Santander Elavon'), ('bitpay', 'TPV Bitpay')], default='', max_length=16, verbose_name='Tipo de TPV'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-04 14:24
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangovirtualpos', '0010_auto_20170329_1227'),
    ]

    operations = [
        migrations.AlterField(
            model_name='vposrefundoperation',
            name='payment',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='refund_operations', to='djangovirtualpos.VPOSPaymentOperation'),
        ),
    ]



# -*- coding: utf-8 -*-

from django import forms
from models import VPOSCeca, VPOSRedsys, VPOSPaypal, VPOSSantanderElavon

from django.conf import settings
from models import VPOS_TYPES


class TrimForm(forms.ModelForm):
    """Django no hace trim por defecto, así que tenemos que crear esta clase
	para hacerlo nosotros de forma explícita a la hora de realizar la
	validación del formulario."""

    def clean(self):
        cleaned_data = super(TrimForm, self).clean()

        for field in self.cleaned_data:
            if isinstance(self.cleaned_data[field], basestring):
                self.cleaned_data[field] = self.cleaned_data[field].strip()

        # Hay que devolver siempre el array "cleaned_data"
        return cleaned_data


# Habrá que añadir un formulario nuevo como este para cada uno de los distintos
# TPVs que se añadan
class VPOSCecaForm(TrimForm):
    """Formulario para el modelo TpvCeca."""

    class Meta:
        model = VPOSCeca
        exclude = ("type", "is_erased")


class VPOSRedsysForm(TrimForm):
    """Formulario para el modelo TpvRedsys."""

    class Meta:
        model = VPOSRedsys
        exclude = ("type", "is_erased")


class VPOSPaypalForm(TrimForm):
    """Formulario para el modelo TpvPaypal."""

    class Meta:
        model = VPOSPaypal
        exclude = ("type", "is_erased")


class VPOSSantanderElavonForm(TrimForm):
    """Formulario para el modelo TpvSantanderElavon."""

    class Meta:
        model = VPOSSantanderElavon
        exclude = ("type", "is_erased")


# Esta variable habrá que actualizarla cada vez que se añada un formulario
# para un TPV
VPOS_FORMS = {
    "ceca": VPOSCecaForm,
    "redsys": VPOSRedsysForm,
    "paypal": VPOSPaypalForm,
    "santanderelavon": VPOSSantanderElavonForm
}


class VPOSTypeForm(forms.Form):
    """Formulario para la selección de tipo de TPV a crear en un paso
	posterior.

	Esta formado por un campo "select" en el que los valores son aquellos
	contenidos en la variable settings.TPVS o todos los disponibles en
	TPV_TYPES en models.py
	"""

    def __init__(self, *args, **kwargs):
        super(VPOSTypeForm, self).__init__(*args, **kwargs)

        if hasattr(settings, "ENABLED_VPOS_LIST") and settings.ENABLED_VPOS_LIST:
            vpos_types = settings.ENABLED_VPOS_LIST
        else:
            vpos_types = VPOS_TYPES

        self.fields["type"] = forms.ChoiceField(choices=vpos_types, required=True, label="Tipo de TPV",
                                                help_text="Tipo de pasarela de pago a crear.")


class DeleteForm(forms.Form):
    """Formulario para confirmación de borrado de elementos"""
    erase = forms.BooleanField(required=False, label="Borrar")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangovirtualpos', '0007_auto_20170320_1757'),
    ]

    operations = [
        migrations.AlterField(
            model_name='vpospaymentoperation',
            name='sale_code',
            field=models.CharField(help_text='C\xf3digo de la venta seg\xfan la aplicaci\xf3n.', max_length=512, verbose_name='C\xf3digo de la venta'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function

import inspect
from django.conf import settings
from django.utils import timezone

import logging


# Prepare debug message
def _prepare_debuglog_message(message, caller_level=2):
    # Por si acaso se le mete una cadena de tipo str,
    # este módulo es capaz de detectar eso y convertirla a UTF8
    if type(message) == str:
        message = unicode(message, "UTF-8")

    # Hora
    now = timezone.now()

    # Contexto desde el que se ha llamado
    curframe = inspect.currentframe()

    # Objeto frame que llamó a dlprint
    calframes = inspect.getouterframes(curframe, caller_level)
    caller_frame = calframes[2][0]
    caller_name = calframes[2][3]

    # Ruta del archivo que llamó a dlprint
    filename_path = caller_frame.f_code.co_filename
    filename = filename_path.split("/")[-1]

    # Obtención del mensaje
    return u"DjangoVirtualPOS: {0} {1} \"{2}\" at {3}:{5} in {6} ({4}:{5})\n".format(
        now.strftime("%Y-%m-%d %H:%M:%S %Z"), settings.DOMAIN, message,
                     filename, filename_path, caller_frame.f_lineno, caller_name
    )


# Prints the debug message
def dlprint(message):
    logger = logging.getLogger("syslog")
    complete_message = _prepare_debuglog_message(message=message, caller_level=3)
    utf8_complete_message = complete_message.encode('UTF-8')
    logger.debug(utf8_complete_message)
    print(utf8_complete_message)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangovirtualpos', '0005_auto_20170314_1057'),
    ]

    operations = [
        migrations.AddField(
            model_name='vpospaymentoperation',
            name='environment',
            field=models.CharField(default='', max_length=255, verbose_name='Entorno del TPV', blank=True, choices=[('testing', 'Pruebas'), ('production', 'Producci\xf3n')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-12-12 16:30
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import re


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='VirtualPointOfSale',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128, verbose_name='Nombre')),
                ('bank_name', models.CharField(max_length=128, verbose_name='Nombre de la entidad bancaria')),
                ('type', models.CharField(choices=[('ceca', 'TPV Virtual - Confederaci\xf3n Espa\xf1ola de Cajas de Ahorros (CECA)'), ('paypal', 'Paypal'), ('redsys', 'TPV Redsys'), ('santanderelavon', 'TPV Santander Elavon')], default='', max_length=16, verbose_name='Tipo de TPV')),
                ('distributor_name', models.CharField(blank=True, help_text='Raz\xf3n social del distribuidor de entradas.', max_length=512, verbose_name='Raz\xf3n social del distribuidor de entradas')),
                ('distributor_cif', models.CharField(blank=True, help_text='C.I.F. del distribuidor de entradas.', max_length=150, verbose_name='CIF del distribuidor de entradas')),
                ('environment', models.CharField(choices=[('testing', 'Pruebas'), ('production', 'Producci\xf3n')], default='testing', help_text="Entorno de ejecuci\xf3n en el que se encuentra el TPV. Una vez que el TPV est\xe9 en entorno de 'producci\xf3n' no cambie a entorno de 'pruebas' a no ser que est\xe9 seguro de lo que hace.", max_length=16, verbose_name='Entorno de ejecuci\xf3n del TPV')),
                ('has_partial_refunds', models.BooleanField(default=False, help_text='Indica si se pueden realizar devoluciones por un importe menor que el total de la venta (por ejemplo, para devolver tickets individuales).', verbose_name='Indica si tiene devoluciones parciales.')),
                ('has_total_refunds', models.BooleanField(default=False, help_text='Indica si se pueden realizar devoluciones por un importe igual al total de la venta.', verbose_name='Indica si tiene devoluciones totales.')),
                ('is_erased', models.BooleanField(default=False, help_text='Indica si el TPV est\xe1 eliminado de forma l\xf3gica.', verbose_name='Indica si el TPV est\xe1 eliminado.')),
            ],
            options={
                'ordering': ['name'],
                'verbose_name': 'virtual point of sale',
                'verbose_name_plural': 'virtual points of sale',
            },
        ),
        migrations.CreateModel(
            name='VPOSPaymentOperation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=6, verbose_name='Coste de la operaci\xf3n')),
                ('description', models.CharField(max_length=512, verbose_name='Descripci\xf3n de la venta')),
                ('url_ok', models.CharField(help_text='URL a la que redirige la pasarela bancaria cuando la compra ha sido un \xe9xito', max_length=255, verbose_name='URL de OK')),
                ('url_nok', models.CharField(help_text='URL a la que redirige la pasarela bancaria cuando la compra ha fallado', max_length=255, verbose_name='URL de NOK')),
                ('operation_number', models.CharField(max_length=255, verbose_name='N\xfamero de operaci\xf3n')),
                ('confirmation_code', models.CharField(max_length=255, null=True, verbose_name='C\xf3digo de confirmaci\xf3n enviado por el banco.')),
                ('confirmation_data', models.TextField(null=True, verbose_name='POST enviado por la pasarela bancaria al confirmar la compra.')),
                ('sale_code', models.CharField(help_text='C\xf3digo de la venta seg\xfan la aplicaci\xf3n.', max_length=512, verbose_name='C\xf3digo de la venta')),
                ('status', models.CharField(choices=[('pending', 'Pending'), ('completed', 'Completed'), ('failed', 'Failed')], max_length=64, verbose_name='Estado del pago')),
                ('creation_datetime', models.DateTimeField(verbose_name='Fecha de creaci\xf3n del objeto')),
                ('last_update_datetime', models.DateTimeField(verbose_name='Fecha de \xfaltima actualizaci\xf3n del objeto')),
                ('type', models.CharField(choices=[('ceca', 'TPV Virtual - Confederaci\xf3n Espa\xf1ola de Cajas de Ahorros (CECA)'), ('paypal', 'Paypal'), ('redsys', 'TPV Redsys'), ('santanderelavon', 'TPV Santander Elavon')], default='', max_length=16, verbose_name='Tipo de TPV')),
                ('virtual_point_of_sale', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, parent_link=True, related_name='+', to='djangovirtualpos.VirtualPointOfSale')),
            ],
        ),
        migrations.CreateModel(
            name='VPOSCeca',
            fields=[
                ('parent', models.OneToOneField(db_column='vpos_id', on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='+', serialize=False, to='djangovirtualpos.VirtualPointOfSale')),
                ('merchant_id', models.CharField(max_length=9, validators=[django.core.validators.MinLengthValidator(9), django.core.validators.MaxLengthValidator(9), django.core.validators.RegexValidator(message='Aseg\xfarese de que todos los caracteres son n\xfameros', regex=re.compile('^\\d*$'))], verbose_name='MerchantID')),
                ('acquirer_bin', models.CharField(max_length=10, validators=[django.core.validators.MinLengthValidator(10), django.core.validators.MaxLengthValidator(10), django.core.validators.RegexValidator(message='Aseg\xfarese de que todos los caracteres son n\xfameros', regex=re.compile('^\\d*$'))], verbose_name='AcquirerBIN')),
                ('terminal_id', models.CharField(max_length=8, validators=[django.core.validators.MinLengthValidator(8), django.core.validators.MaxLengthValidator(8), django.core.validators.RegexValidator(message='Aseg\xfarese de que todos los caracteres son n\xfameros', regex=re.compile('^\\d*$'))], verbose_name='TerminalID')),
                ('encryption_key_testing', models.CharField(max_length=10, validators=[django.core.validators.MinLengthValidator(8), django.core.validators.MaxLengthValidator(10)], verbose_name='Encryption Key para el entorno de pruebas')),
                ('encryption_key_production', models.CharField(blank=True, max_length=10, validators=[django.core.validators.MinLengthValidator(8), django.core.validators.MaxLengthValidator(10)], verbose_name='Encryption Key para el entorno de producci\xf3n')),
                ('operation_number_prefix', models.CharField(blank=True, max_length=20, validators=[django.core.validators.MinLengthValidator(0), django.core.validators.MaxLengthValidator(20), django.core.validators.RegexValidator(message='Aseg\xfarese de s\xf3lo use caracteres alfanum\xe9ricos', regex=re.compile('^[A-Za-z0-9]*$'))], verbose_name='Prefijo del n\xfamero de operaci\xf3n')),
            ],
            bases=('djangovirtualpos.virtualpointofsale',),
        ),
        migrations.CreateModel(
            name='VPOSPaypal',
            fields=[
                ('parent', models.OneToOneField(db_column='vpos_id', on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='+', serialize=False, to='djangovirtualpos.VirtualPointOfSale')),
                ('API_username', models.CharField(max_length=60, verbose_name='API_username')),
                ('API_password', models.CharField(max_length=60, verbose_name='API_password')),
                ('API_signature', models.CharField(max_length=60, verbose_name='API_signature')),
                ('Version', models.CharField(max_length=3, verbose_name='Version')),
            ],
            bases=('djangovirtualpos.virtualpointofsale',),
        ),
        migrations.CreateModel(
            name='VPOSRedsys',
            fields=[
                ('parent', models.OneToOneField(db_column='vpos_id', on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='+', serialize=False, to='djangovirtualpos.VirtualPointOfSale')),
                ('merchant_code', models.CharField(max_length=9, verbose_name='MerchantCode')),
                ('merchant_response_url', models.URLField(help_text='Confirmation URL that will be used by the virtual POS', max_length=64, verbose_name='MerchantURL')),
                ('terminal_id', models.CharField(max_length=3, verbose_name='TerminalID')),
                ('encryption_key_testing', models.CharField(default=None, max_length=20, null=True, verbose_name='Encryption Key para el entorno de pruebas (OBSOLETO)')),
                ('encryption_key_production', models.CharField(default=None, max_length=20, null=True, verbose_name='Encryption Key para el entorno de producci\xf3n (OBSOLETO)')),
                ('encryption_key_testing_sha256', models.CharField(default=None, max_length=64, null=True, verbose_name='Encryption Key SHA-256 para el entorno de pruebas')),
                ('encryption_key_production_sha256', models.CharField(default=None, max_length=64, null=True, verbose_name='Encryption Key SHA-256 para el entorno de producci\xf3n')),
                ('operation_number_prefix', models.CharField(blank=True, max_length=3, validators=[django.core.validators.MinLengthValidator(0), django.core.validators.MaxLengthValidator(3), django.core.validators.RegexValidator(message='Aseg\xfarese de s\xf3lo use caracteres num\xe9ricos', regex=re.compile('^\\d+$'))], verbose_name='Prefijo del n\xfamero de operaci\xf3n')),
            ],
            bases=('djangovirtualpos.virtualpointofsale',),
        ),
        migrations.CreateModel(
            name='VPOSSantanderElavon',
            fields=[
                ('parent', models.OneToOneField(db_column='vpos_id', on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='+', serialize=False, to='djangovirtualpos.VirtualPointOfSale')),
                ('merchant_id', models.CharField(max_length=50, validators=[django.core.validators.MinLengthValidator(1), django.core.validators.MaxLengthValidator(50), django.core.validators.RegexValidator(message='Aseg\xfarese de que todos los caracteres son alfanum\xe9ricos', regex=re.compile('^[a-zA-Z0-9]*$'))], verbose_name='MerchantID')),
                ('merchant_response_url', models.URLField(help_text='Confirmation URL that will be used by the virtual POS', max_length=64, verbose_name='MerchantURL')),
                ('account', models.CharField(max_length=30, validators=[django.core.validators.MinLengthValidator(0), django.core.validators.MaxLengthValidator(30), django.core.validators.RegexValidator(message='Aseg\xfarese de que todos los caracteres son alfanum\xe9ricos', regex=re.compile('^[a-zA-Z0-9.]*$'))], verbose_name='Account')),
                ('encryption_key', models.CharField(max_length=64, validators=[django.core.validators.MinLengthValidator(8), django.core.validators.MaxLengthValidator(10)], verbose_name='Clave secreta de cifrado')),
                ('operation_number_prefix', models.CharField(blank=True, max_length=20, validators=[django.core.validators.MinLengthValidator(0), django.core.validators.MaxLengthValidator(20), django.core.validators.RegexValidator(message='Aseg\xfarese de s\xf3lo use caracteres alfanum\xe9ricos', regex=re.compile('^[A-Za-z0-9]*$'))], verbose_name='Prefijo del n\xfamero de operaci\xf3n')),
            ],
            bases=('djangovirtualpos.virtualpointofsale',),
        ),
    ]



# -*- coding: utf-8 -*-

from django.forms import forms
from django.forms.models import ModelMultipleChoiceField
from django.contrib.admin.widgets import FilteredSelectMultiple
from django.db.models import Count

from djangovirtualpos import models


class VPOSField(ModelMultipleChoiceField):
    """ Campo personalizado para la selección de TPVs """

    def __init__(self, queryset=None, required=True,
                 widget=None, label="VPOSs", initial=None,
                 help_text="TPVs que se podrán utilizar para realizar el pago online.", *args, **kwargs):

        if queryset is None:
            queryset = models.VPOS.objects.filter(is_deleted=False)

        if widget is None:
            widget = FilteredSelectMultiple("", False)

        super(VPOSField, self).__init__(
            queryset=queryset,
            required=required,
            widget=widget,
            label=label,
            initial=initial,
            help_text=help_text,
            *args, **kwargs)

    def clean(self, value):
        if value:
            # Si se reciben valores (id's de Tpvs), cargarlos para comprobar si son todos del mismo tipo.
            # Construimos un ValuesQuerySet con sólo el campo "type", hacemos la cuenta y ordenamos en orden descendente para comprobar el primero (esto es como hacer un "group_by" y un count)
            # Si el primero es mayor que 1 mostramos el error oportuno.
            count = models.VPOS.objects.filter(id__in=value).values("type").annotate(Count("type")).order_by(
                "-type__count")
            if count[0]["type__count"] > 1:
                raise forms.ValidationError("Asegúrese de no seleccionar más de un TPV del tipo '{0}'".format(
                    dict(models.VPOS_TYPES)[count[0]["type"]]))
        return super(VPOSField, self).clean(value)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangovirtualpos', '0004_auto_20170313_1401'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='virtualpointofsale',
            options={'ordering': ['name'], 'verbose_name': 'virtual point of sale', 'verbose_name_plural': 'virtual points of sale', 'permissions': (('view_virtualpointofsale', 'View Virtual Points of Sale'),)},
        ),
        migrations.AlterField(
            model_name='virtualpointofsale',
            name='distributor_cif',
            field=models.CharField(help_text='C.I.F. del distribuidor.', max_length=150, verbose_name='CIF del distribuidor', blank=True),
        ),
        migrations.AlterField(
            model_name='virtualpointofsale',
            name='distributor_name',
            field=models.CharField(help_text='Raz\xf3n social del distribuidor.', max_length=512, verbose_name='Raz\xf3n social del distribuidor', blank=True),
        ),
    ]
